@@ -9,20 +9,28 @@ import os.path
 import string
 import sys
 import datetime
-import cgi
 import calendar
 
 try:
-    import debug
+    from html import escape
+except ImportError:
+    from cgi import escape
+
+try:
+    from xml2rfc import debug
+    debug.debug = True
     assert debug
 except ImportError:
     pass
 
 # Local libs
 import xml2rfc
+from xml2rfc.util.name import short_author_name_parts
+from xml2rfc.util.unicode import expand_unicode_element
 from xml2rfc.writers.base import BaseRfcWriter, default_options
 
 
+
 class HtmlRfcWriter(BaseRfcWriter):
     """ Writes to an HTML file.
 
@@ -34,7 +42,7 @@ class HtmlRfcWriter(BaseRfcWriter):
     }
 
     def __init__(self, xmlrfc, quiet=None, options=default_options,
-                               date=datetime.date.today(), templates_dir=None):
+                               date=None, templates_dir=None):
         if not quiet is None:
             options.quiet = quiet
         BaseRfcWriter.__init__(self, xmlrfc, options=options)
@@ -44,7 +52,7 @@ class HtmlRfcWriter(BaseRfcWriter):
         self.cref_counter = 0
         
         # Initialize templates directory
-        self.templates_dir = templates_dir or \
+        self.template_dir = templates_dir or \
                              os.path.join(os.path.dirname(xml2rfc.__file__),
                                           'templates')
 
@@ -52,9 +60,8 @@ class HtmlRfcWriter(BaseRfcWriter):
         self.templates = {}
         for filename in ['base.html',
                          'address_card.html']:
-            file = open(os.path.join(self.templates_dir, filename), 'r')
-            self.templates[filename] = string.Template(file.read())
-            file.close()
+            with open(os.path.join(self.template_dir, filename), 'r') as file:
+                self.templates[filename] = string.Template(file.read())
         # Buffers to aggregate various elements before processing template
         self.buffers = {'front': [],
                         'body': [],
@@ -301,6 +308,18 @@ class HtmlRfcWriter(BaseRfcWriter):
                 elem.tail = element.tail
             return [elem]
 
+    def _expand_u(self, element):
+        try:
+            text = expand_unicode_element(element)
+        except (RuntimeError, ValueError) as exc:
+            text = ''
+            xml2rfc.log.error('%s'%exc)
+        if text:
+            span = E.SPAN(text)
+            span.set('class', 'unicode')
+            return [span]
+        return []
+
     # -----------------------------------------
     # Base writer overrides
     # -----------------------------------------
@@ -405,6 +424,9 @@ class HtmlRfcWriter(BaseRfcWriter):
             if child.tag in ['xref', 'eref', 'iref', 'cref', 'spanx']:
                 for element in self._expand_ref(child):
                     current.append(element)
+            elif child.tag == 'u':
+                for element in self._expand_u(child):
+                    current.append(element)
             elif child.tag == 'vspace':
                 br = E.BR()
                 current.append(br)
@@ -548,8 +570,8 @@ class HtmlRfcWriter(BaseRfcWriter):
             for j, author in enumerate(authors):
                 organization = author.find('organization')
                 email = author.find('address/email')
-                surname = author.attrib.get('surname')
-                initials = self.get_initials(author)
+                initials, surname = short_author_name_parts(author)
+                initials = self.get_initials(author) or initials or ''
                 a = None
                 if j == len(authors) - 1 and len(authors) > 1:
                     last.tail = ' and '
@@ -776,9 +798,9 @@ class HtmlRfcWriter(BaseRfcWriter):
         else:
             docName = self.r.attrib.get('docName', '') 
         description = ''
-        abs_t = self.r.find('front/abstract/t')
-        if abs_t is not None and abs_t.text:
-            description = abs_t.text
+        abs = self.r.find('front/abstract')
+        if abs != None:
+            description = ''.join(abs.itertext())
         keywords = self.r.findall('front/keyword')
         keyword_list = [keyword.text for keyword in keywords if keyword.text]
         generator = "xml2rfc version %s - https://tools.ietf.org/tools/xml2rfc" % xml2rfc.__version__
@@ -798,7 +820,9 @@ class HtmlRfcWriter(BaseRfcWriter):
                 docDate += "-%02d" % int(date.attrib.get('day'))
 
         # Build author string
-        authors = self._format_author_string(self.r.findall('front/author'))
+        authors = self.r.findall('front/author')
+        authors = [ a for a in authors if a.get('role') != 'contributor' ]
+        authors = self._format_author_string(authors)
 
         # Run through base template, store in main output string
         subs = { 
@@ -806,13 +830,13 @@ class HtmlRfcWriter(BaseRfcWriter):
                  'background':      background_image,
 
                  # HTML-escaped values
-                 'docName':         cgi.escape(docName, quote=True),
-                 'docDate':         cgi.escape(docDate, quote=True),
-                 'description':     cgi.escape(description, quote=True),
-                 'generator':       cgi.escape(generator, quote=True),
-                 'authors':         cgi.escape(authors, quote=True),
-                 'keywords':        cgi.escape(', '.join(keyword_list), quote=True),
-                 'title':           cgi.escape(title),
+                 'docName':         escape(docName, quote=True),
+                 'docDate':         escape(docDate, quote=True),
+                 'description':     escape(description, quote=True),
+                 'generator':       escape(generator, quote=True),
+                 'authors':         escape(authors, quote=True),
+                 'keywords':        escape(', '.join(keyword_list), quote=True),
+                 'title':           escape(title, quote=False),
                  
                  # Replace buffers
                  'front':           ''.join(self.buffers['front']),
@@ -822,7 +846,7 @@ class HtmlRfcWriter(BaseRfcWriter):
                  'toc_rows':        ''.join(self.buffers['toc_rows'])
                 }
         for key in subs.keys():
-            subs[key] = subs[key].encode('ascii', 'xmlcharrefreplace').decode('ascii')
+            subs[key] = subs[key].replace('\uE060', '\u2060').encode('ascii', 'xmlcharrefreplace').decode('ascii')
         self.output = self.templates['base.html'].substitute(subs)
 
     def post_process_lines(self, lines):
